import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { IonicModule, PopoverController } from '@ionic/angular';

import { CalendarEditorPopover } from './calendar-editor.popover';
import { CalendarService } from 'src/app/services/calendar.service';
import { MockTranslatePipe } from 'src/test/mock-translate.pipe';
import { of, throwError } from 'rxjs';

describe('CalendarEditorPopover', () => {

  let popoverController: jasmine.SpyObj<PopoverController>;
  let calendarService: jasmine.SpyObj<CalendarService>;
  let component: CalendarEditorPopover;
  let fixture: ComponentFixture<CalendarEditorPopover>;

  beforeEach(waitForAsync(() => {
    const popoverControllerSpy = jasmine.createSpyObj('PopoverController', ['dismiss']);
    const calendarServiceSpy = jasmine.createSpyObj('CalendarService', ['saveCalendar']);

    TestBed.configureTestingModule({
      declarations: [ CalendarEditorPopover, MockTranslatePipe ],
      imports: [IonicModule.forRoot()],
      providers: [
        { provide: PopoverController, useValue: popoverControllerSpy },
        { provide: CalendarService, useValue: calendarServiceSpy },
      ],
    }).compileComponents();

    popoverController = TestBed.inject(PopoverController) as jasmine.SpyObj<PopoverController>;
    calendarService = TestBed.inject(CalendarService) as jasmine.SpyObj<CalendarService>;

    fixture = TestBed.createComponent(CalendarEditorPopover);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }));

  it('should create', done => {
    expect(component).toBeTruthy();
    done();
  });

  it('should show new calendar title when empty calendar', done => {
    const title = fixture.nativeElement.querySelector('ion-header ion-title');
    expect(title).toBeTruthy();
    expect(title.textContent).toBe('calendars.new_calendar');
    done();
  });

  it('should show edit calendar title when editing calendar', done => {
    component.calendar = { id: 1, name: 'any name' };
    fixture.detectChanges();
    const title = fixture.nativeElement.querySelector('ion-header ion-title');
    expect(title).toBeTruthy();
    expect(title.textContent).toBe('calendars.edit_calendar');
    done();
  });

  it('should have calendar name input', done => {
    let label = fixture.nativeElement.querySelector('ion-item#name ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('calendars.calendar_name');
    let input = fixture.nativeElement.querySelector('ion-item#name ion-input input');
    expect(input).toBeTruthy();
    expect(input.value).toBe('');
    done();
  });

  it('should have functional cancel button', done => {
    spyOn(component, 'cancel');
    const button = fixture.nativeElement.querySelector('ion-button#cancel');
    expect(button).toBeTruthy();
    expect(button.textContent).toBe('commons.cancel');
    button.click();
    expect(component.cancel).toHaveBeenCalled();
    done();
  });

  it('should have functional save button', done => {
    spyOn(component, 'save');
    const button = fixture.nativeElement.querySelector('ion-button#save');
    expect(button).toBeTruthy();
    expect(button.textContent).toBe('commons.save');
    button.click();
    expect(component.save).toHaveBeenCalled();
    done();
  });

  it('should dismiss popover on cancel', done => {
    component.cancel();
    expect(calendarService.saveCalendar).toHaveBeenCalledTimes(0);
    expect(popoverController.dismiss).toHaveBeenCalledOnceWith(null, 'cancel');
    done();
  });

  it('should save calendar and dismiss popover on save', done => {
    calendarService.saveCalendar.and.returnValue(of());
    component.save();
    expect(calendarService.saveCalendar).toHaveBeenCalledOnceWith(component.calendar);
    expect(popoverController.dismiss).toHaveBeenCalledOnceWith(null, 'save');
    done();
  });

  it('should dismiss popover with error message on save error', done => {
    calendarService.saveCalendar.and.returnValue(throwError('any error'));
    component.save();
    expect(calendarService.saveCalendar).toHaveBeenCalledOnceWith(component.calendar);
    expect(popoverController.dismiss).toHaveBeenCalledOnceWith('any error', 'error');
    done();
  });

});
