import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { PopoverController, IonicModule } from '@ionic/angular';
import { of, throwError } from 'rxjs';
import { Coworker } from 'src/app/models/coworker';
import { CalendarService } from 'src/app/services/calendar.service';
import { CoworkerService } from 'src/app/services/coworker.service';
import { MockTranslatePipe } from 'src/test/mock-translate.pipe';
import { CoworkerEditorPopover } from './coworker-editor.popover';

describe('CoworkerEditorPopover', () => {

  let popoverController: jasmine.SpyObj<PopoverController>;
  let calendarService: jasmine.SpyObj<CalendarService>;
  let coworkerService: jasmine.SpyObj<CoworkerService>;
  let component: CoworkerEditorPopover;
  let fixture: ComponentFixture<CoworkerEditorPopover>;

  beforeEach(waitForAsync(() => {
    const popoverControllerSpy = jasmine.createSpyObj('PopoverController', ['dismiss']);
    const calendarServiceSpy = jasmine.createSpyObj('CalendarService', ['getCalendars']);
    calendarServiceSpy.getCalendars.and.returnValue(of())
    const coworkerServiceSpy = jasmine.createSpyObj('CoworkerService', ['saveCoworker']);

    TestBed.configureTestingModule({
      declarations: [ CoworkerEditorPopover, MockTranslatePipe ],
      imports: [IonicModule.forRoot()],
      providers: [
        { provide: PopoverController, useValue: popoverControllerSpy },
        { provide: CalendarService, useValue: calendarServiceSpy },
        { provide: CoworkerService, useValue: coworkerServiceSpy },
      ],
    }).compileComponents();

    popoverController = TestBed.inject(PopoverController) as jasmine.SpyObj<PopoverController>;
    calendarService = TestBed.inject(CalendarService) as jasmine.SpyObj<CalendarService>;
    coworkerService = TestBed.inject(CoworkerService) as jasmine.SpyObj<CoworkerService>;

    fixture = TestBed.createComponent(CoworkerEditorPopover);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }));

  it('should create', done => {
    expect(component).toBeTruthy();
    done();
  });

  it('should show new coworker title when empty coworker', done => {
    const title = fixture.nativeElement.querySelector('ion-header ion-title');
    expect(title).toBeTruthy();
    expect(title.textContent).toBe('coworkers.new_coworker');
    done();
  });

  it('should show edit coworker title when editing coworker', done => {
    component.coworker = new Coworker();
    component.coworker.id = 1;
    fixture.detectChanges();
    const title = fixture.nativeElement.querySelector('ion-header ion-title');
    expect(title).toBeTruthy();
    expect(title.textContent).toBe('coworkers.edit_coworker');
    done();
  });

  it('should have coworker name input', done => {
    let label = fixture.nativeElement.querySelector('ion-item#name ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('coworkers.coworker_name');
    let input = fixture.nativeElement.querySelector('ion-item#name ion-input input');
    expect(input).toBeTruthy();
    expect(input.value).toBe('');
    done();
  });

  it('should have coworker calendars select', done => {
    let label = fixture.nativeElement.querySelector('ion-item#calendars ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('coworkers.coworker_calendars');
    let input = fixture.nativeElement.querySelector('ion-item#calendars ion-select');
    expect(input).toBeTruthy();
    done();
  });

  it('should have functional cancel button', done => {
    spyOn(component, 'cancel');
    const button = fixture.nativeElement.querySelector('ion-button#cancel');
    expect(button).toBeTruthy();
    expect(button.textContent).toBe('commons.cancel');
    button.click();
    expect(component.cancel).toHaveBeenCalled();
    done();
  });

  it('should have functional save button', done => {
    spyOn(component, 'save');
    const button = fixture.nativeElement.querySelector('ion-button#save');
    expect(button).toBeTruthy();
    expect(button.textContent).toBe('commons.save');
    button.click();
    expect(component.save).toHaveBeenCalled();
    done();
  });

  it('should dismiss popover on cancel', done => {
    component.cancel();
    expect(coworkerService.saveCoworker).toHaveBeenCalledTimes(0);
    expect(popoverController.dismiss).toHaveBeenCalledOnceWith(null, 'cancel');
    done();
  });

  it('should save coworker and dismiss popover on save', done => {
    coworkerService.saveCoworker.and.returnValue(of());
    component.save();
    expect(coworkerService.saveCoworker).toHaveBeenCalledOnceWith(component.coworker);
    expect(popoverController.dismiss).toHaveBeenCalledOnceWith(null, 'save');
    done();
  });

  it('should dismiss popover with error message on save error', done => {
    coworkerService.saveCoworker.and.returnValue(throwError('any error'));
    component.save();
    expect(coworkerService.saveCoworker).toHaveBeenCalledOnceWith(component.coworker);
    expect(popoverController.dismiss).toHaveBeenCalledOnceWith('any error', 'error');
    done();
  });

});
