import { ApplicationRef, Component, Input, OnInit } from '@angular/core';
import { PopoverController } from '@ionic/angular';
import { Calendar } from 'src/app/interfaces/calendar';
import { Coworker } from 'src/app/models/coworker';
import { CalendarService } from 'src/app/services/calendar.service';
import { CoworkerService } from 'src/app/services/coworker.service';

@Component({
  selector: 'app-coworker-editor-popover',
  templateUrl: './coworker-editor.popover.html',
  styleUrls: ['./coworker-editor.popover.scss'],
})
export class CoworkerEditorPopover implements OnInit {

  calendars: Calendar[] = [];
  @Input() coworker: Coworker = new Coworker();

  constructor(
    private popover: PopoverController,
    private app: ApplicationRef,
    private calendarService: CalendarService,
    private coworkerService: CoworkerService,
  ) {
    this.calendarService.getCalendars().subscribe({
      next: calendars => {
        this.calendars = calendars;
        this.app.tick();
      }
    });
  }

  ngOnInit() {
  }

  cancel() {
    this.popover.dismiss(null, 'cancel');
  }

  compareCalendars(c1: Calendar, c2: Calendar | Calendar[]) {
    if (Array.isArray(c2)) {
      return c2.some(c => c.id === c1.id);
    }
    return c1 && c2 ? c1.id === c2.id : c1 === c2;
  }

  save() {
    this.coworkerService.saveCoworker(this.coworker).subscribe({
      complete: () => this.popover.dismiss(null, 'save'),
      error: (error) => this.popover.dismiss(error, 'error'),
    });
  }

}
