import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { IonicModule, PopoverController } from '@ionic/angular';

import { HomeEditPopover } from './home-edit.popover';

describe('HomeEditPopover', () => {

  let popoverController: jasmine.SpyObj<PopoverController>;
  let component: HomeEditPopover;
  let fixture: ComponentFixture<HomeEditPopover>;

  beforeEach(waitForAsync(() => {
    const popoverControllerSpy = jasmine.createSpyObj('PopoverController', ['dismiss']);

    TestBed.configureTestingModule({
      declarations: [ HomeEditPopover ],
      imports: [IonicModule.forRoot()],
      providers: [
        { provide: PopoverController, useValue: popoverControllerSpy },
      ]
    }).compileComponents();

    popoverController = TestBed.inject(PopoverController) as jasmine.SpyObj<PopoverController>;

    fixture = TestBed.createComponent(HomeEditPopover);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }));

  it('should create', done => {
    expect(component).toBeTruthy();
    done();
  });

  it('should show options', done => {
    const options = fixture.nativeElement.querySelectorAll('ion-item ion-label');
    expect(options.length).toEqual(6);
    expect(options[0].textContent).toBe('home.edit.add_turn');
    expect(options[1].textContent).toBe('home.edit.add_note');
    expect(options[2].textContent).toBe('home.edit.switch_turn');
    expect(options[3].textContent).toBe('home.edit.remove_turn');
    expect(options[4].textContent).toBe('home.edit.copy_pattern');
    expect(options[5].textContent).toBe('home.edit.close');
    done();
  });

  it('should dismiss popover on add turn', done => {
    spyOn(component, 'addTurn').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-item#add-turn');
    expect(item).toBeTruthy();
    item.click();
    expect(component.addTurn).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledWith(null, 'addTurn');
    done();
  });

  it('should dismiss popover on swap turn', done => {
    spyOn(component, 'swapTurn').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-item#swap-turn');
    expect(item).toBeTruthy();
    item.click();
    expect(component.swapTurn).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledWith(null, 'swapTurn');
    done();
  });

  it('should dismiss popover on delete turn', done => {
    spyOn(component, 'deleteTurn').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-item#delete-turn');
    expect(item).toBeTruthy();
    item.click();
    expect(component.deleteTurn).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledWith(null, 'deleteTurn');
    done();
  });

  it('should dismiss popover on close', done => {
    spyOn(component, 'close').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-item#close');
    expect(item).toBeTruthy();
    item.click();
    expect(component.close).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledWith(null, 'close');
    done();
  });
});
