import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { IonicModule, PopoverController } from '@ionic/angular';
import { of } from 'rxjs';
import { CalendarService } from 'src/app/services/calendar.service';
import { CALENDARS } from 'src/test/mock-calendars';

import { HomeViewPopover } from './home-view.popover';

describe('HomeViewPopover', () => {

  let popoverController: jasmine.SpyObj<PopoverController>;
  let calendarService: jasmine.SpyObj<CalendarService>;
  let component: HomeViewPopover;
  let fixture: ComponentFixture<HomeViewPopover>;

  beforeEach(waitForAsync(() => {
    const popoverControllerSpy = jasmine.createSpyObj('PopoverController', ['dismiss']);
    const calendarServiceSpy = jasmine.createSpyObj('CalendarService', ['getCalendars']);

    TestBed.configureTestingModule({
      declarations: [ HomeViewPopover ],
      imports: [IonicModule.forRoot()],
      providers: [
        { provide: PopoverController, useValue: popoverControllerSpy },
        { provide: CalendarService, useValue: calendarServiceSpy },
      ],
    }).compileComponents();

    popoverController = TestBed.inject(PopoverController) as jasmine.SpyObj<PopoverController>;
    calendarService = TestBed.inject(CalendarService) as jasmine.SpyObj<CalendarService>;
    calendarService.getCalendars.and.returnValue(of(CALENDARS));

    fixture = TestBed.createComponent(HomeViewPopover);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }));

  it('should create', done => {
    expect(component).toBeTruthy();
    done()
  });

  it('should have view list', done => {
    const list = fixture.nativeElement.querySelector('ion-list#view-list');
    expect(list).toBeTruthy();
    const listHeader = fixture.nativeElement.querySelector('ion-list#view-list ion-list-header');
    expect(listHeader).toBeTruthy();
    expect(listHeader.textContent).toBe('home.view.view');
    const options = fixture.nativeElement.querySelectorAll('ion-list#view-list ion-radio-group ion-label');
    expect(options.length).toEqual(2);
    expect(options[0].textContent).toBe('home.view.month');
    expect(options[1].textContent).toBe('home.view.week');
    done();
  });

  it('should have calendars list', done => {
    const list = fixture.nativeElement.querySelector('ion-list#calendars-list');
    expect(list).toBeTruthy();
    const listHeader = fixture.nativeElement.querySelector('ion-list#calendars-list ion-list-header');
    expect(listHeader).toBeTruthy();
    expect(listHeader.textContent).toBe('home.view.calendars');
    const calendars = fixture.nativeElement.querySelectorAll('ion-list#calendars-list ion-checkbox');
    expect(calendars.length).toEqual(2);
    done();
  });

  it('should have functional accept button', done => {
    spyOn(component, 'accept').and.callThrough();
    const button = fixture.nativeElement.querySelector('ion-button');
    expect(button).toBeTruthy();
    expect(button.textContent).toBe('home.view.accept');
    button.click();
    expect(component.accept).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalled();
    done();
  });
});
