import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { IonicModule, PopoverController } from '@ionic/angular';

import { OptionsPopover } from './options.popover';

describe('OptionsPopover', () => {

  let popoverController: jasmine.SpyObj<PopoverController>;
  let component: OptionsPopover;
  let fixture: ComponentFixture<OptionsPopover>;

  beforeEach(waitForAsync(() => {
    const popoverControllerSpy = jasmine.createSpyObj('PopoverController', ['dismiss']);

    TestBed.configureTestingModule({
      declarations: [ OptionsPopover ],
      imports: [IonicModule.forRoot()],
      providers: [
        { provide: PopoverController, useValue: popoverControllerSpy },
      ],
    }).compileComponents();

    popoverController = TestBed.inject(PopoverController) as jasmine.SpyObj<PopoverController>;

    fixture = TestBed.createComponent(OptionsPopover);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }));

  it('should create', done => {
    expect(component).toBeTruthy();
    done();
  });

  it('should show options', done => {
    const options = fixture.nativeElement.querySelectorAll('ion-item ion-label');
    expect(options.length).toEqual(3);
    expect(options[0].textContent).toBe('commons.edit');
    expect(options[1].textContent).toBe('commons.delete');
    expect(options[2].textContent).toBe('commons.cancel');
    done();
  });

  it('should dismiss popover on edit', done => {
    spyOn(component, 'edit').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-item#edit');
    expect(item).toBeTruthy();
    item.click();
    expect(component.edit).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledWith(null, 'edit');
    done();
  });

  it('should dismiss popover on delete', done => {
    spyOn(component, 'delete').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-item#delete');
    expect(item).toBeTruthy();
    item.click();
    expect(component.delete).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledWith(null, 'delete');
    done();
  });

  it('should dismiss popover on cancel', done => {
    spyOn(component, 'cancel').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-item#cancel');
    expect(item).toBeTruthy();
    item.click();
    expect(component.cancel).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledWith(null, 'cancel');
    done();
  });
});
