import { DatePipe } from '@angular/common';
import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { IonicModule, PopoverController } from '@ionic/angular';
import { of } from 'rxjs';
import { CalendarService } from 'src/app/services/calendar.service';
import { CoworkerService } from 'src/app/services/coworker.service';
import { CALENDARS } from 'src/test/mock-calendars';
import { COWORKERS } from 'src/test/mock-coworkers';

import { SwapsFilterPopover } from './swaps-filter.popover';

describe('SwapsFilterPopover', () => {

  let popoverController: jasmine.SpyObj<PopoverController>;
  let calendarService: jasmine.SpyObj<CalendarService>;
  let coworkerService: jasmine.SpyObj<CoworkerService>;
  let component: SwapsFilterPopover;
  let fixture: ComponentFixture<SwapsFilterPopover>;

  beforeEach(waitForAsync(() => {
    const popoverControllerSpy = jasmine.createSpyObj('PopoverController', ['dismiss']);
    const datePipeSpy = jasmine.createSpyObj('DatePipe', ['transform']);
    const calendarServiceSpy = jasmine.createSpyObj('CalendarService', ['getCalendars']);
    const coworkerServiceSpy = jasmine.createSpyObj('CoworkerService', ['getCoworkers']);

    TestBed.configureTestingModule({
      declarations: [ SwapsFilterPopover ],
      imports: [IonicModule.forRoot()],
      providers: [
        { provide: PopoverController, useValue: popoverControllerSpy },
        { provide: DatePipe, useValue: datePipeSpy },
        { provide: CalendarService, useValue: calendarServiceSpy },
        { provide: CoworkerService, useValue: coworkerServiceSpy },
      ],
    }).compileComponents();

    popoverController = TestBed.inject(PopoverController) as jasmine.SpyObj<PopoverController>;
    calendarService = TestBed.inject(CalendarService) as jasmine.SpyObj<CalendarService>;
    calendarService.getCalendars.and.returnValue(of(CALENDARS));
    coworkerService = TestBed.inject(CoworkerService) as jasmine.SpyObj<CoworkerService>;
    coworkerService.getCoworkers.and.returnValue(of(COWORKERS));

    fixture = TestBed.createComponent(SwapsFilterPopover);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }));

  it('should create', done => {
    expect(component).toBeTruthy();
    done();
  });

  it('should show swaps filter title', done => {
    const title = fixture.nativeElement.querySelector('ion-header ion-title');
    expect(title).toBeTruthy();
    expect(title.textContent).toBe('swaps_filter.swaps_filter_title');
    done();
  });

  it('should have from date datetime', done => {
    const label = fixture.nativeElement.querySelector('ion-item#from-date ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('swaps_filter.from_date');
    const datetime = fixture.nativeElement.querySelector('ion-item#from-date ion-datetime');
    expect(datetime).toBeTruthy();
    done();
  });

  it('should have to date datetime', done => {
    const label = fixture.nativeElement.querySelector('ion-item#to-date ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('swaps_filter.to_date');
    const datetime = fixture.nativeElement.querySelector('ion-item#to-date ion-datetime');
    expect(datetime).toBeTruthy();
    done();
  });


  it('should have calendar select', done => {
    const label = fixture.nativeElement.querySelector('ion-item#calendar > ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('swaps_filter.calendar');
    const options = fixture.nativeElement.querySelectorAll('ion-item#calendar ion-select-option');
    expect(options.length).toEqual(CALENDARS.length + 1);
    expect(options[0].textContent).toBe("swaps_filter.all_calendars");
    for (const [index, value] of CALENDARS.entries()) {
      expect(options[index + 1].textContent).toBe(value.name);
    }
    done();
  });


  it('should have coworker select', done => {
    const label = fixture.nativeElement.querySelector('ion-item#coworker > ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('swaps_filter.coworker');
    const options = fixture.nativeElement.querySelectorAll('ion-item#coworker ion-select-option');
    expect(options.length).toEqual(COWORKERS.length + 1);
    expect(options[0].textContent).toBe("swaps_filter.all_coworkers");
    for (const [index, value] of COWORKERS.entries()) {
      expect(options[index + 1].textContent).toBe(value.name);
    }
    done();
  });

  it('should dismiss popover on apply', done => {
    spyOn(component, 'apply').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-button#apply');
    expect(item).toBeTruthy();
    item.click();
    expect(component.apply).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalled();
    done();
  });
});
