import { DatePipe } from '@angular/common';
import { ApplicationRef, Component, OnInit } from '@angular/core';
import { PopoverController } from '@ionic/angular';
import { Calendar } from 'src/app/interfaces/calendar';
import { Coworker } from 'src/app/models/coworker';
import { CalendarService } from 'src/app/services/calendar.service';
import { CoworkerService } from 'src/app/services/coworker.service';

@Component({
  selector: 'app-swaps-filter-popover',
  templateUrl: './swaps-filter.popover.html',
  styleUrls: ['./swaps-filter.popover.scss'],
})
export class SwapsFilterPopover implements OnInit {

  calendars: Calendar[] = [];
  coworkers: Coworker[] = [];
  filterCalendar: Calendar = null;
  filterCoworker: Coworker = null;
  fromDate: string = null;
  toDate: string = null;

  constructor(
    private popover: PopoverController,
    private app: ApplicationRef,
    private calendarService: CalendarService,
    private coworkerService: CoworkerService,
    private datePipe: DatePipe,
  ) {
  }

  ngOnInit() {
    this.loadCalendars();
    this.loadCoworkers();
  }

  apply() {
    this.popover.dismiss({
      filterCalendar: this.filterCalendar,
      filterCoworker: this.filterCoworker,
      fromDate: this.datePipe.transform(this.fromDate, 'yyyy-MM-dd'),
      toDate: this.datePipe.transform(this.toDate, 'yyyy-MM-dd'),
    }, "apply");
  }

  private loadCalendars() {
    this.calendarService.getCalendars().subscribe({
      next: calendars => {
        this.calendars = calendars;
        if (this.filterCalendar != null) {
          this.filterCalendar = calendars.find((it) => it.id === this.filterCalendar.id);
        }
        this.app.tick();
      }
    });
  }

  private loadCoworkers() {
    this.coworkerService.getCoworkers().subscribe({
      next: coworkers => {
        this.coworkers = coworkers;
        if (this.filterCoworker != null) {
          this.filterCoworker = coworkers.find((it) => it.id === this.filterCoworker.id);
        }
        this.app.tick();
      }
    });
  }

}
