import { DatePipe } from '@angular/common';
import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { IonicModule, PopoverController } from '@ionic/angular';
import { of } from 'rxjs';
import { CalendarService } from 'src/app/services/calendar.service';
import { ShiftTypeService } from 'src/app/services/shift-type.service';
import { ShiftService } from 'src/app/services/shift.service';
import { CALENDARS } from 'src/test/mock-calendars';
import { SHIFT_TYPES } from 'src/test/mock-shift-types';
import { MockTranslatePipe } from 'src/test/mock-translate.pipe';

import { TurnCreationPopover } from './turn-creation.popover';

describe('TurnCreationPopover', () => {

  let popoverController: jasmine.SpyObj<PopoverController>;
  let calendarService: jasmine.SpyObj<CalendarService>;
  let shiftService: jasmine.SpyObj<ShiftService>;
  let shiftTypeService: jasmine.SpyObj<ShiftTypeService>;
  let component: TurnCreationPopover;
  let fixture: ComponentFixture<TurnCreationPopover>;

  beforeEach(waitForAsync(() => {
    const popoverControllerSpy = jasmine.createSpyObj('PopoverController', ['dismiss']);
    const datePipeSpy = jasmine.createSpyObj('DatePipe', ['transform']);
    const calendarServiceSpy = jasmine.createSpyObj('CalendarService', ['getCalendars']);
    const shiftServiceSpy = jasmine.createSpyObj('ShiftService', ['saveShift']);
    const shiftTypeServiceSpy = jasmine.createSpyObj('ShiftTypeService', ['getShiftTypes']);

    TestBed.configureTestingModule({
      declarations: [ TurnCreationPopover, MockTranslatePipe ],
      imports: [IonicModule.forRoot()],
      providers: [
        { provide: PopoverController, useValue: popoverControllerSpy },
        { provide: DatePipe, useValue: datePipeSpy },
        { provide: CalendarService, useValue: calendarServiceSpy },
        { provide: ShiftService, useValue: shiftServiceSpy },
        { provide: ShiftTypeService, useValue: shiftTypeServiceSpy },
      ],
    }).compileComponents();

    popoverController = TestBed.inject(PopoverController) as jasmine.SpyObj<PopoverController>;
    calendarService = TestBed.inject(CalendarService) as jasmine.SpyObj<CalendarService>;
    calendarService.getCalendars.and.returnValue(of(CALENDARS));
    shiftService = TestBed.inject(ShiftService) as jasmine.SpyObj<ShiftService>;
    shiftService.saveShift.and.returnValue(of());
    shiftTypeService = TestBed.inject(ShiftTypeService) as jasmine.SpyObj<ShiftTypeService>;
    shiftTypeService.getShiftTypes.and.returnValue(of(SHIFT_TYPES));

    fixture = TestBed.createComponent(TurnCreationPopover);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }));

  it('should create', done => {
    expect(component).toBeTruthy();
    done();
  });

  it('should show turn creation title', done => {
    const title = fixture.nativeElement.querySelector('ion-header ion-title');
    expect(title).toBeTruthy();
    expect(title.textContent).toBe('turn_creation.add_turn');
    done();
  });

  it('should have shift date datetime', done => {
    const label = fixture.nativeElement.querySelector('ion-item#shift-date ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('turn_creation.shift_date');
    const datetime = fixture.nativeElement.querySelector('ion-item#shift-date ion-datetime');
    expect(datetime).toBeTruthy();
    done();
  });

  it('should have calendar select', done => {
    const label = fixture.nativeElement.querySelector('ion-item#calendar > ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('turn_creation.calendar');
    const options = fixture.nativeElement.querySelectorAll('ion-item#calendar ion-select-option');
    expect(options.length).toEqual(CALENDARS.length);
    for (const [index, value] of CALENDARS.entries()) {
      expect(options[index].textContent).toBe(value.name);
    }
    done();
  });

  it('should have disabled shift type select', done => {
    const label = fixture.nativeElement.querySelector('ion-item#shift-type > ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('turn_creation.shift_type');
    const select = fixture.nativeElement.querySelector('ion-item#shift-type ion-select');
    expect(select).toBeTruthy();
    expect(select.disabled).toBeTruthy();
    done();
  });

  it('should dismiss popover on cancel', done => {
    spyOn(component, 'cancel').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-button#cancel');
    expect(item).toBeTruthy();
    expect(item.textContent).toBe('turn_creation.cancel');
    item.click();
    expect(component.cancel).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledOnceWith(null, "cancel");
    done();
  });

  it('should dismiss popover on save', done => {
    spyOn(component, 'save').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-button#save');
    expect(item).toBeTruthy();
    expect(item.textContent).toBe('turn_creation.save');
    item.click();
    expect(component.save).toHaveBeenCalled();
    expect(shiftService.saveShift).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledOnceWith(null, "save");
    done();
  });
});
