import { DatePipe } from '@angular/common';
import { ApplicationRef, Component, OnInit } from '@angular/core';
import { PopoverController } from '@ionic/angular';
import { Calendar } from 'src/app/interfaces/calendar';
import { Shift } from 'src/app/models/shift';
import { ShiftType } from 'src/app/models/shift-type';
import { CalendarService } from 'src/app/services/calendar.service';
import { ShiftService } from 'src/app/services/shift.service';
import { ShiftTypeService } from 'src/app/services/shift-type.service';

@Component({
  selector: 'app-turn-creation-popover',
  templateUrl: './turn-creation.popover.html',
  styleUrls: ['./turn-creation.popover.scss'],
})
export class TurnCreationPopover implements OnInit {

  shift: Shift = new Shift();
  calendars: Calendar[] = [];
  calendarShiftTypes: ShiftType[] = [];

  private shiftTypes: ShiftType[] = [];

  constructor(
    private popover: PopoverController,
    private app: ApplicationRef,
    private calendarService: CalendarService,
    private shiftService: ShiftService,
    private shiftTypeService: ShiftTypeService,
    private datePipe: DatePipe,
  ) {
    this.shift.date = this.datePipe.transform(new Date(), 'yyyy-MM-dd');
    this.loadCalendars();
    this.loadShiftTypes();
  }

  ngOnInit() {}

  cancel() {
    this.popover.dismiss(null, "cancel");
  }

  onCalendarChange() {
    this.calendarShiftTypes = this.shiftTypes.filter((it) => it.isEnabledForCalendar(this.shift.calendar));
    if (this.shift.type != null && !this.shift.type.isEnabledForCalendar(this.shift.calendar)) {
      this.shift.type = null;
    }
  }

  save() {
    this.shift.date = this.datePipe.transform(this.shift.date, 'yyyy-MM-dd');
    this.shiftService.saveShift(this.shift).subscribe({
      complete: () => this.popover.dismiss(null, "save")
    });
  }

  private loadCalendars() {
    this.calendarService.getCalendars().subscribe({
      next: calendars => {
        this.calendars = calendars;
        this.app.tick();
      }
    });
  }

  private loadShiftTypes() {
    this.shiftTypeService.getShiftTypes().subscribe({
      next: shiftTypes => {
        this.shiftTypes = shiftTypes;
        this.app.tick();
      }
    });
  }

}
