import { DatePipe } from '@angular/common';
import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { IonicModule, PopoverController } from '@ionic/angular';
import { of } from 'rxjs';
import { CalendarService } from 'src/app/services/calendar.service';
import { ShiftTypeService } from 'src/app/services/shift-type.service';
import { ShiftService } from 'src/app/services/shift.service';
import { CALENDARS } from 'src/test/mock-calendars';

import { TurnDeletionPopover } from './turn-deletion.popover';

describe('TurnDeletionPopover', () => {

  let popoverController: jasmine.SpyObj<PopoverController>;
  let calendarService: jasmine.SpyObj<CalendarService>;
  let shiftService: jasmine.SpyObj<ShiftService>;
  let component: TurnDeletionPopover;
  let fixture: ComponentFixture<TurnDeletionPopover>;

  beforeEach(waitForAsync(() => {
    const popoverControllerSpy = jasmine.createSpyObj('PopoverController', ['dismiss']);
    const datePipeSpy = jasmine.createSpyObj('DatePipe', ['transform']);
    const calendarServiceSpy = jasmine.createSpyObj('CalendarService', ['getCalendars']);
    const shiftServiceSpy = jasmine.createSpyObj('ShiftService', ['deleteShift', 'findAllShiftsByCalendarAndDate']);

    TestBed.configureTestingModule({
      declarations: [ TurnDeletionPopover ],
      imports: [IonicModule.forRoot()],
      providers: [
        { provide: PopoverController, useValue: popoverControllerSpy },
        { provide: DatePipe, useValue: datePipeSpy },
        { provide: CalendarService, useValue: calendarServiceSpy },
        { provide: ShiftService, useValue: shiftServiceSpy },
      ],
    }).compileComponents();

    popoverController = TestBed.inject(PopoverController) as jasmine.SpyObj<PopoverController>;
    calendarService = TestBed.inject(CalendarService) as jasmine.SpyObj<CalendarService>;
    calendarService.getCalendars.and.returnValue(of(CALENDARS));
    shiftService = TestBed.inject(ShiftService) as jasmine.SpyObj<ShiftService>;
    shiftService.deleteShift.and.returnValue(of());

    fixture = TestBed.createComponent(TurnDeletionPopover);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }));

  it('should create', done => {
    expect(component).toBeTruthy();
    done();
  });

  it('should show turn deletion title', done => {
    const title = fixture.nativeElement.querySelector('ion-header ion-title');
    expect(title).toBeTruthy();
    expect(title.textContent).toBe('turn_deletion.delete_turn');
    done();
  });

  it('should have date datetime', done => {
    const label = fixture.nativeElement.querySelector('ion-item#date ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('turn_deletion.date');
    const datetime = fixture.nativeElement.querySelector('ion-item#date ion-datetime');
    expect(datetime).toBeTruthy();
    done();
  });

  it('should have calendar select', done => {
    const label = fixture.nativeElement.querySelector('ion-item#calendar ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('turn_deletion.calendar');
    const options = fixture.nativeElement.querySelectorAll('ion-item#calendar ion-select-option');
    expect(options.length).toEqual(CALENDARS.length);
    for (const [index, value] of CALENDARS.entries()) {
      expect(options[index].textContent).toBe(value.name);
    }
    done();
  });

  it('should have disabled shift select', done => {
    const label = fixture.nativeElement.querySelector('ion-item#shift ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('turn_deletion.shift');
    const select = fixture.nativeElement.querySelector('ion-item#shift ion-select');
    expect(select).toBeTruthy();
    expect(select.disabled).toBeTruthy();
    done();
  });

  it('should dismiss popover on cancel', done => {
    spyOn(component, 'cancel').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-button#cancel');
    expect(item).toBeTruthy();
    expect(item.textContent).toBe('commons.cancel');
    item.click();
    expect(component.cancel).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledOnceWith(null, "cancel");
    done();
  });

  it('should dismiss popover on delete', done => {
    spyOn(component, 'delete').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-button#delete');
    expect(item).toBeTruthy();
    expect(item.textContent).toBe('commons.delete');
    item.click();
    expect(component.delete).toHaveBeenCalled();
    expect(shiftService.deleteShift).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledOnceWith(null, "delete");
    done();
  });
});
