import { DatePipe } from '@angular/common';
import { ApplicationRef, Component, OnInit } from '@angular/core';
import { PopoverController } from '@ionic/angular';
import { Calendar } from 'src/app/interfaces/calendar';
import { Shift } from 'src/app/models/shift';
import { CalendarService } from 'src/app/services/calendar.service';
import { ShiftService } from 'src/app/services/shift.service';

@Component({
  selector: 'app-turn-deletion-popover',
  templateUrl: './turn-deletion.popover.html',
  styleUrls: ['./turn-deletion.popover.scss'],
})
export class TurnDeletionPopover implements OnInit {

  calendars: Calendar[] = [];
  date: string = this.datePipe.transform(new Date(), 'yyyy-MM-dd');
  dayShifts: Shift[] = [];
  selectedCalendar: Calendar = null;
  selectedShift: Shift = null;

  constructor(
    private popover: PopoverController,
    private app: ApplicationRef,
    private calendarService: CalendarService,
    private shiftService: ShiftService,
    private datePipe: DatePipe,
  ) {
    this.loadCalendars();
  }

  ngOnInit() {}

  cancel() {
    this.popover.dismiss(null, "cancel");
  }

  delete() {
    this.shiftService.deleteShift(this.selectedShift).subscribe({
      complete: () => this.popover.dismiss(null, "delete")
    });
  }

  onCalendarChange() {
    this.loadDayShifts();
  }

  onDateChange() {
    this.loadDayShifts();
  }

  private loadCalendars() {
    this.calendarService.getCalendars().subscribe({
      next: calendars => {
        this.calendars = calendars;
        this.app.tick();
      }
    });
  }

  private loadDayShifts() {
    if (this.selectedCalendar != null && this.date != null) {
      const date = this.datePipe.transform(this.date, 'yyyy-MM-dd');
      this.shiftService.findAllShiftsByCalendarAndDate(this.selectedCalendar, date).subscribe({
        next: shifts => {
          this.dayShifts = shifts;
          this.app.tick();
        }
      });
    }
  }

}
