import { DatePipe } from '@angular/common';
import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { IonicModule, PopoverController } from '@ionic/angular';
import { of } from 'rxjs';
import { CalendarService } from 'src/app/services/calendar.service';
import { CoworkerService } from 'src/app/services/coworker.service';
import { ShiftTypeService } from 'src/app/services/shift-type.service';
import { ShiftService } from 'src/app/services/shift.service';
import { CALENDARS } from 'src/test/mock-calendars';
import { COWORKERS } from 'src/test/mock-coworkers';
import { SHIFT_TYPES } from 'src/test/mock-shift-types';

import { TurnSwapPopover } from './turn-swap.popover';

describe('TurnSwapPopover', () => {

  let popoverController: jasmine.SpyObj<PopoverController>;
  let calendarService: jasmine.SpyObj<CalendarService>;
  let coworkerService: jasmine.SpyObj<CoworkerService>;
  let shiftService: jasmine.SpyObj<ShiftService>;
  let shiftTypeService: jasmine.SpyObj<ShiftTypeService>;
  let component: TurnSwapPopover;
  let fixture: ComponentFixture<TurnSwapPopover>;

  beforeEach(waitForAsync(() => {
    const popoverControllerSpy = jasmine.createSpyObj('PopoverController', ['dismiss']);
    const datePipeSpy = jasmine.createSpyObj('DatePipe', ['transform']);
    const calendarServiceSpy = jasmine.createSpyObj('CalendarService', ['getCalendars']);
    const coworkerServiceSpy = jasmine.createSpyObj('CoworkerService', ['getCoworkers']);
    const shiftServiceSpy = jasmine.createSpyObj('ShiftService', ['saveShift', 'findShiftsByCalendarAndDate']);
    const shiftTypeServiceSpy = jasmine.createSpyObj('ShiftTypeService', ['getShiftTypes']);

    TestBed.configureTestingModule({
      declarations: [ TurnSwapPopover ],
      imports: [IonicModule.forRoot()],
      providers: [
        { provide: PopoverController, useValue: popoverControllerSpy },
        { provide: DatePipe, useValue: datePipeSpy },
        { provide: CalendarService, useValue: calendarServiceSpy },
        { provide: CoworkerService, useValue: coworkerServiceSpy },
        { provide: ShiftService, useValue: shiftServiceSpy },
        { provide: ShiftTypeService, useValue: shiftTypeServiceSpy },
      ],
    }).compileComponents();

    popoverController = TestBed.inject(PopoverController) as jasmine.SpyObj<PopoverController>;
    calendarService = TestBed.inject(CalendarService) as jasmine.SpyObj<CalendarService>;
    calendarService.getCalendars.and.returnValue(of(CALENDARS));
    coworkerService = TestBed.inject(CoworkerService) as jasmine.SpyObj<CoworkerService>;
    coworkerService.getCoworkers.and.returnValue(of(COWORKERS));
    shiftService = TestBed.inject(ShiftService) as jasmine.SpyObj<ShiftService>;
    shiftTypeService = TestBed.inject(ShiftTypeService) as jasmine.SpyObj<ShiftTypeService>;
    shiftTypeService.getShiftTypes.and.returnValue(of(SHIFT_TYPES));

    fixture = TestBed.createComponent(TurnSwapPopover);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }));

  it('should create', done => {
    expect(component).toBeTruthy();
    done();
  });

  it('should show turn swap title', done => {
    const title = fixture.nativeElement.querySelector('ion-header ion-title');
    expect(title).toBeTruthy();
    expect(title.textContent).toBe('turn_swap.swap_turn');
    done();
  });

  it('should have direction select', done => {
    const label = fixture.nativeElement.querySelector('ion-item#direction ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('turn_swap.direction');
    const options = fixture.nativeElement.querySelectorAll('ion-item#direction ion-select-option');
    expect(options.length).toEqual(2);
    expect(options[0].textContent).toBe('turn_swap.user_to_coworker');
    expect(options[1].textContent).toBe('turn_swap.coworker_to_user');
    done();
  });

  it('should have coworker select', done => {
    const label = fixture.nativeElement.querySelector('ion-item#coworker ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('turn_swap.coworker');
    const options = fixture.nativeElement.querySelectorAll('ion-item#coworker ion-select-option');
    expect(options.length).toEqual(COWORKERS.length);
    for (const [index, value] of COWORKERS.entries()) {
      expect(options[index].textContent).toBe(value.name);
    }
    done();
  });

  it('should have shift date datetime', done => {
    const label = fixture.nativeElement.querySelector('ion-item#shift-date ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('turn_swap.shift_date');
    const datetime = fixture.nativeElement.querySelector('ion-item#shift-date ion-datetime');
    expect(datetime).toBeTruthy();
    done();
  });

  it('should have calendar select', done => {
    const label = fixture.nativeElement.querySelector('ion-item#calendar ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('turn_swap.calendar');
    const options = fixture.nativeElement.querySelectorAll('ion-item#calendar ion-select-option');
    expect(options.length).toEqual(CALENDARS.length);
    for (const [index, value] of CALENDARS.entries()) {
      expect(options[index].textContent).toBe(value.name);
    }
    done();
  });

  it('should dismiss popover on cancel', done => {
    spyOn(component, 'cancel').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-button#cancel');
    expect(item).toBeTruthy();
    expect(item.textContent).toBe('commons.cancel');
    item.click();
    expect(component.cancel).toHaveBeenCalled();
    expect(popoverController.dismiss).toHaveBeenCalledOnceWith(null, "cancel");
    done();
  });

  it('should dismiss popover on save', done => {
    spyOn(component, 'save').and.callThrough();
    const item = fixture.nativeElement.querySelector('ion-button#save');
    expect(item).toBeTruthy();
    expect(item.textContent).toBe('commons.save');
    item.click();
    expect(component.save).toHaveBeenCalled();
    done();
  });
});
