import { DatePipe } from '@angular/common';
import { ApplicationRef, Component, OnInit } from '@angular/core';
import { PopoverController } from '@ionic/angular';
import { Calendar } from 'src/app/interfaces/calendar';
import { Shift } from 'src/app/models/shift';
import { ShiftType } from 'src/app/models/shift-type';
import { CalendarService } from 'src/app/services/calendar.service';
import { ShiftService } from 'src/app/services/shift.service';
import { ShiftTypeService } from 'src/app/services/shift-type.service';
import { Swap } from 'src/app/models/swap';
import { CoworkerService } from 'src/app/services/coworker.service';

@Component({
  selector: 'app-turn-swap-popover',
  templateUrl: './turn-swap.popover.html',
  styleUrls: ['./turn-swap.popover.scss'],
})
export class TurnSwapPopover implements OnInit {

  newShift: Shift = new Shift();
  calendars: Calendar[] = [];
  calendarShiftTypes: ShiftType[] = [];
  coworkers: Calendar[] = [];
  dayShifts: Shift[] = [];
  swap: Swap = new Swap();

  private shiftTypes: ShiftType[] = [];

  constructor(
    private popover: PopoverController,
    private app: ApplicationRef,
    private calendarService: CalendarService,
    private coworkerService: CoworkerService,
    private shiftService: ShiftService,
    private shiftTypeService: ShiftTypeService,
    private datePipe: DatePipe,
  ) {
    this.newShift.date = this.datePipe.transform(new Date(), 'yyyy-MM-dd');
    this.loadCalendars();
    this.loadCoworkers();
    this.loadShiftTypes();
  }

  ngOnInit() {}

  cancel() {
    this.popover.dismiss(null, "cancel");
  }

  onCalendarChange() {
    this.calendarShiftTypes = this.shiftTypes.filter((it) => it.isEnabledForCalendar(this.newShift.calendar));
    if (this.newShift.type != null && !this.newShift.type.isEnabledForCalendar(this.newShift.calendar)) {
      this.newShift.type = null;
    }
    this.loadDayShifts();
  }

  onDateChange() {
    this.loadDayShifts();
  }

  onSwapDirectionChange() {
    this.loadDayShifts();
  }

  save() {
    switch (this.swap.direction.toString()) {
      case 'COWORKER_TO_USER':
        this.newShift.date = this.datePipe.transform(this.newShift.date, 'yyyy-MM-dd');
        this.newShift.swap = this.swap;
        this.swap.shift = this.newShift;
        this.shiftService.saveShift(this.newShift).subscribe({
          complete: () => this.popover.dismiss(null, "save")
        });
        break;
      case 'USER_TO_COWORKER':
        this.swap.shift.swap = this.swap;
        this.shiftService.saveShift(this.swap.shift).subscribe({
          complete: () => this.popover.dismiss(null, "save")
        });
        break;
    }
  }

  private loadCalendars() {
    this.calendarService.getCalendars().subscribe({
      next: calendars => {
        this.calendars = calendars;
        this.app.tick();
      }
    });
  }

  private loadCoworkers() {
    this.coworkerService.getCoworkers().subscribe({
      next: coworkers => {
        this.coworkers = coworkers;
        this.app.tick();
      }
    });
  }

  private loadDayShifts() {
    if (this.newShift.calendar != null && this.newShift.date != null) {
      const date = this.datePipe.transform(this.newShift.date, 'yyyy-MM-dd');
      this.shiftService.findShiftsByCalendarAndDate(this.newShift.calendar, date).subscribe({
        next: shifts => {
          this.dayShifts = shifts;
          this.app.tick();
        }
      });
    }
  }

  private loadShiftTypes() {
    this.shiftTypeService.getShiftTypes().subscribe({
      next: shiftTypes => {
        this.shiftTypes = shiftTypes;
        this.app.tick();
      }
    });
  }

}
