import { Shift } from "./shift";

export class Day {

  date: Date;
  shifts: Shift[] = [];

  constructor(date: Date) {
    this.date = date
  }

  isSameDate(date: Date): boolean {
    return this.date.toDateString() === date.toDateString();
  }

  isSameMonth(date: Date): boolean {
    return this.date.getMonth() === date.getMonth();
  }

  getClass(month: Date): String {
    return this.getMonthClass(month) + this.getWeekClass() + this.getDayClass();
  }

  private getDayClass() {
    const todayClass = (this.date.toDateString() === new Date().toDateString()) ? " today" : "";
    const firstDayClass = (this.date.getDate() === 1) ? " first-day" : "";
    const nextDay = new Date(this.date.getFullYear(), this.date.getMonth(), this.date.getDate() + 1);
    const lastDayClass = (nextDay.getDate() === 1) ? " last-day" : "";
    return firstDayClass + lastDayClass + todayClass;
  }

  private getMonthClass(month: Date) {
    const thisAtStartOfMonth = new Date(this.date.getFullYear(), this.date.getMonth(), 1);
    const otherAtStartOfMonth = new Date(month.getFullYear(), month.getMonth(), 1);
    if (thisAtStartOfMonth.getTime() < otherAtStartOfMonth.getTime()) {
      return "previous-month";
    }
    if (thisAtStartOfMonth.getTime() > otherAtStartOfMonth.getTime()) {
      return "next-month";
    }
    return "current-month";
  }

  private getWeekClass() {
    return (this.date.getDate() < 8) ? " first-week" : "";
  }

}
