import { Calendar } from "../interfaces/calendar";
import { Reminder } from "../interfaces/reminder";

export class ShiftType {

  id: number;
  name: string;
  initials: string;
  fromHour: string = "12:00";
  toHour: string = "12:00";
  hours: number;
  price: number;
  color: string;
  calendars: Calendar[] = [];
  reminders: Reminder[] = [];

  getContrastColor(): string {
    return (this.getRelativeLuminance() > 0.179) ? "#000000" : "#FFFFFF";
  }

  isEnabledForCalendar(calendar: Calendar): boolean {
    return this.calendars.some((it) => it.id === calendar.id);
  }

  private getRelativeLuminance(): number {
    const r = this.getRelativeLuminanceComponent(this.color.substring(1, 3));
    const g = this.getRelativeLuminanceComponent(this.color.substring(3, 5));
    const b = this.getRelativeLuminanceComponent(this.color.substring(5));
    return 0.2126 * r + 0.7152 * g + 0.0722 * b;
  }

  private getRelativeLuminanceComponent(hex: string): number {
    const c = parseInt(hex, 16) / 255.0;
    if (c <= 0.03928) {
      return c / 12.92;
    }
    return Math.pow((c + 0.055) / 1.055, 2.4);
  }

}
