import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { IonicModule } from '@ionic/angular';
import { TranslateService } from '@ngx-translate/core';

import { of } from 'rxjs';

import { CalendarsPage } from './calendars.page';
import { CalendarService } from 'src/app/services/calendar.service';
import { CALENDARS } from 'src/test/mock-calendars';
import { MockTranslateService } from 'src/test/mock-translate.service';

describe('CalendarsPage', () => {

  let calendarService: jasmine.SpyObj<CalendarService>;
  let component: CalendarsPage;
  let fixture: ComponentFixture<CalendarsPage>;

  beforeEach(waitForAsync(() => {
    const calendarServiceSpy = jasmine.createSpyObj('CalendarService', ['deleteCalendar', 'getCalendars', 'saveCalendar']);

    TestBed.configureTestingModule({
      declarations: [ CalendarsPage ],
      imports: [IonicModule.forRoot()],
      providers: [
        { provide: CalendarService, useValue: calendarServiceSpy },
        { provide: TranslateService, useClass: MockTranslateService },
      ],
    }).compileComponents();

    calendarService = TestBed.inject(CalendarService) as jasmine.SpyObj<CalendarService>;
    calendarService.getCalendars.and.returnValue(of(CALENDARS));

    fixture = TestBed.createComponent(CalendarsPage);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }));

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should load calendars', () => {
    expect(calendarService.getCalendars.calls.count()).toBe(1);
    expect(component.calendars).toBe(CALENDARS);
    const items = fixture.nativeElement.querySelectorAll('#calendars-list ion-item');
    expect(items.length).toEqual(2);
    expect(items[0].textContent).toBe('Hospital');
    expect(items[1].textContent).toBe('Nursing home');
  });

  it('should have an add calendar button', async () => {
    const button = fixture.nativeElement.querySelector('#actions ion-button');
    expect(button).toBeTruthy();
    const label = fixture.nativeElement.querySelector('#actions ion-label');
    expect(label).toBeTruthy();
    expect(label.textContent).toBe('calendars.add_calendar');
  });

});
