import { ApplicationRef, Component, NgZone, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Calendar } from 'src/app/interfaces/calendar';
import { ShiftType } from 'src/app/models/shift-type';
import { CalendarService } from 'src/app/services/calendar.service';
import { ShiftTypeService } from 'src/app/services/shift-type.service';

@Component({
  selector: 'app-shift-type-editor',
  templateUrl: './shift-type-editor.page.html',
  styleUrls: ['./shift-type-editor.page.scss'],
})
export class ShiftTypeEditorPage implements OnInit {

  calendars: Calendar[] = [];
  shiftType: ShiftType = new ShiftType();

  constructor(
    private app: ApplicationRef,
    private router: Router,
    private calendarService: CalendarService,
    private shiftTypeService: ShiftTypeService,
    private ngZone: NgZone,
  ) {
    let shiftType = new ShiftType();
    if (this.router.getCurrentNavigation().extras.state) {
      shiftType = this.router.getCurrentNavigation().extras.state.shiftType;
    }
    this.calendarService.getCalendars().subscribe({
      next: calendars => {
        this.calendars = calendars;
        this.shiftType = shiftType;
        this.app.tick();
      }
    });
  }

  ngOnInit() {
  }

  cancel() {
    this.ngZone.run(() => this.router.navigate(['shift-types']))
  }

  compareCalendars(c1: Calendar, c2: Calendar | Calendar[]) {
    if (Array.isArray(c2)) {
      return c2.some(c => c.id === c1.id);
    }
    return c1 && c2 ? c1.id === c2.id : c1 === c2;
  }

  save() {
    this.shiftTypeService.saveShiftType(this.shiftType).subscribe({
      complete: () => this.ngZone.run(() => this.router.navigate(['shift-types']))
    });
  }

}
