import { Injectable } from '@angular/core';

import { Observable, Subject, throwError } from 'rxjs';

import { Calendar } from '../interfaces/calendar';
import { DatabaseService } from './database.service';

@Injectable({
  providedIn: 'root'
})
export class CalendarService {

  constructor(private database: DatabaseService) {}

  deleteCalendar(calendar: Calendar): Observable<void> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql('DELETE FROM calendars WHERE id = ?', [calendar.id], () => result.complete(), () => result.error("calendars.error.could_not_delete"))
      )
    );
    return result;
  }

  getCalendars(): Observable<Calendar[]> {
    const calendars = new Subject<Calendar[]>();
    this.database.getDatabase().subscribe((db) => {
      db.transaction((tx) => {
        tx.executeSql('SELECT * FROM calendars', [], (_, result) => {
          calendars.next(this.readCalendars(result.rows));
        });
      });
    });
    return calendars;
  }

  saveCalendar(calendar: Calendar): Observable<void> {
    calendar.name = calendar.name?.trim();
    if (!calendar.name) {
      return throwError("calendars.error.empty_name");
    }
    if (!calendar.id) {
      return this.save(calendar);
    }
    return this.update(calendar);
  }

  private save(calendar: Calendar): Observable<void> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql('INSERT INTO calendars (name) VALUES (?)', [calendar.name], () => result.complete(), (_, err) => result.error("calendars.error.could_not_save"))
      )
    );
    return result;
  }

  private update(calendar: Calendar): Observable<any> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql('UPDATE calendars SET name = ? WHERE id = ?', [calendar.name, calendar.id], () => result.complete(), () => result.error("calendars.error.could_not_update"))
      )
    );
    return result;
  }

  private readCalendars(rows): Calendar[] {
    const calendars = [];
    for (let i = 0; i < rows.length; i++) {
      calendars.push(rows.item(i) as Calendar)
    }
    return calendars;
  }

}
